/*
 * Defines.cpp
 *
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */
/*
 * This file defines a bunch of macros and defines that the
 * C-preprocessor uses on .Java files when generating a .java file.
 *
 * See the Makefile (java/Makefiles/Preprocess.mk) for just how that
 * happens.
 *
 * Note: All .Java files are "dependent" on this file (in make terms)
 * so making a change here will cause the world to need recompilation.
 */

#define JANOS_ROOT_PKG                 edu.utah.janos
#define JANOS_NODEOS_PKG               JANOS_ROOT_PKG##.nodeos
#define JANOS_IO_PKG                   JANOS_ROOT_PKG##.io
#define JANOS_UTIL_PKG                 JANOS_ROOT_PKG##.util
#define JANOS_ROUTE_PKG                JANOS_ROOT_PKG##.route
#define JANOS_PROTOCOLS_PKG            JANOS_ROOT_PKG##.protocols
#define JANOS_SERVICES_PKG             JANOS_ROOT_PKG##.services
#define JANOS_APPS_PKG                 JANOS_ROOT_PKG##.apps
#define JANOS_BENCHMARKS_PKG           JANOS_ROOT_PKG##.benchmarks
#define JANOS_TESTS_PKG                JANOS_ROOT_PKG##.tests
#define JANOS_TESTS_NODEOS_PKG         JANOS_ROOT_PKG##.tests.nodeos

#define JVM_ROOT_PKG                   edu.utah.janosvm
#define JVM_SYS_PKG                    JVM_ROOT_PKG##.sys
#define JVM_RESOURCES_PKG              JVM_ROOT_PKG##.resources

#define JANOS_NODEOS_PKG_S              "edu.utah.janos.nodeos"
#define JANOS_IO_PKG_S                  "edu.utah.janos.io"
#define JANOS_UTIL_PKG_S                "edu.utah.janos.util"
#define JANOS_ROUTE_PKG_S               "edu.utah.janos.route"
#define JANOS_PROTOCOLS_PKG_S           "edu.utah.janos.protocols"
#define JANOS_SERVICES_PKG_S            "edu.utah.janos.services"
#define JANOS_APPS_PKG_S                "edu.utah.janos.apps"
#define JANOS_BENCHMARKS_PKG_S          "edu.utah.janos.benchmarks"
#define JANOS_TESTS_PKG_S               "edu.utah.janos.tests"
#define JANOS_TESTS_NODEOS_PKG_S        "edu.utah.janos.tests.nodeos"

#define JVM_SYS_PKG_S                   "edu.utah.janosvm.sys"
#define JVM_RESOURCES_PKG_S             "edu.utah.janosvm.resources"

/*
 * Convenience macros.
 */

#define UNSIGNED_BYTE2INT(x)	((x) & 0xff)

#define UNSIGNED_SHORT2INT(x)	((x) & 0xffff)

#define EXTRACT_SHORT(array, offset) 			\
	(UNSIGNED_SHORT2INT(				\
	 (UNSIGNED_BYTE2INT(array[(offset)]) << 8) | 	\
	 (UNSIGNED_BYTE2INT(array[(int)(offset)+1]))))

#define EXTRACT_INT(array, offset) 				\
	((UNSIGNED_BYTE2INT(array[(offset)]) << 24) | 		\
	 (UNSIGNED_BYTE2INT(array[(offset) + 1]) << 16) | 	\
	 (UNSIGNED_BYTE2INT(array[(offset) + 2]) << 8) | 	\
	 (UNSIGNED_BYTE2INT(array[(offset) + 3])))

/*
 * So we may label our package-private methods with a token
 */
#ifndef PACKAGE
# define PACKAGE 
#endif


/*
 * EXPOSED: Make all class members/fields visible.  For testing!
 */
#ifdef EXPOSED
# undef PACKAGE
# define private public
# define PACKAGE public
# define protected public
#endif

#ifndef DEBUG
# ifndef NDEBUG
# error DEBUG or NDEBUG must be defined on the preprocessor command line
# endif
#endif

/*
 * DEBUG: Provide all sorts of magical macros.
 */
#ifdef DEBUG
# define DEBUG_ON true
# define IMPORT_DEBUG import JANOS_NODEOS_PKG.NATIVE_DEBUG; import JANOS_NODEOS_PKG.Debug;
# define IMPORT_TRACE import JANOS_NODEOS_PKG.Trace;
# define DEBUGONLY(x) x
# define assert(x) JANOS_NODEOS_PKG.Assert.assertCheck(x, #x, __FILE__, __LINE__)
# define require(x) JANOS_NODEOS_PKG.Assert.requireCheck(x, #x, __FILE__, __LINE__)
# define ensure(x) JANOS_NODEOS_PKG.Assert.ensureCheck(x, #x, __FILE__, __LINE__)
# define assertLocked(x) JANOS_NODEOS_PKG.Assert.assertLockedCheck(x, #x, __FILE__, __LINE__)
# define FATALEXCEPTION(EXEP, MSG) throw JANOS_NODEOS_PKG.Debug.fatalException(EXEP, MSG)
# define panic(STR) throw JANOS_NODEOS_PKG.Debug.doPanic(STR)
# define NOTIMPLEMENTED(MSG) throw JANOS_NODEOS_PKG.Debug.notImplemented(MSG)
/* XXX Make trace flags a per-flow variable. */
# define TRACE(FLAGS, STR) { if (JANOS_NODEOS_PKG.Trace.check(FLAGS)) { JANOS_NODEOS_PKG.Debug.pTrace(__FILE__, __LINE__, STR); } }
# define CHECK_NATIVE_NODEOS						\
	static 								\
	{								\
		assert(BufferHandle.NativeNodeOSID == NATIVE_NODEOS);	\
	}
#endif

/*
 * NDEBUG: Don't actually provide any magical macros.
 */
#ifdef NDEBUG
# define DEBUG_ON false
# define IMPORT_DEBUG 
# define IMPORT_TRACE 
# define DEBUGONLY(x)
# define assert(x)
# define require(x)
# define ensure(x)
# define assertLocked(x)
# define FATALEXCEPTION(EXEP, MSG) throw new InternalError(EXEP.getMessage() + MSG)
# define panic(STR) throw new InternalError(STR)
# define NOTIMPLEMENTED(MSG) throw new InternalError(MSG)
# define TRACE(x, y)
# define CHECK_NATIVE_NODEOS
#endif

/*
 * PROFILED: Define the PROFILESTAGE() macro
 */
#ifdef PROFILED
# define IMPORT_PROFILER  import kaffe.management.XProfiler;
# define PROFILESTAGE(x) { Node.println("Saving profile stage "+ x ); XProfiler.stage(x); }
#else
# define IMPORT_PROFILER
# define PROFILESTAGE(x)
#endif


