###
### Preprocess.mk
###
### Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
### All rights reserved.
###
### This file is part of the Janos Java NodeOS software.
### 
### Permission to use, copy, modify, distribute, and sell this software and
### its documentation is hereby granted without fee, provided that the above
### copyright notice and this permission/disclaimer notice is retained in all
### copies or modified versions, and that both notices appear in supporting
### documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
### WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
### THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
### PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
### ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
### 
### Users are requested, but not required, to send to csl-dist@cs.utah.edu any
### improvements that they make and grant Univ. of Utah redistribution rights.
###

###
### Preprocessor stuff.  
###
### This file must define the makefile variables PREPROCESS and DOC_MAGIC_SUBST.
### They will be used like this: 
###     '$(PREPROCESS) [infile] >> [outfile]'
### Also, DefinePackage.mk depends on CPP_GLOBAL_MACROS as a dependency.
###
### This file should define no targets (other than mkdebug::)
###
### The following variables can be expected to be defined:
###  BUILD, LIBSTYLE, PACKAGE_PREFIX, PACKAGE_PREFIX_DIR, DEST_DIR, SRC_DIR
###
### XXX Watch out for name clashes, only one namespace in make.
###

# The CPP definition used throughout the Java code
CPP_GLOBAL_MACROS:=$(SRC_DIR)/NativeNodeOS.cpp $(SRC_DIR)/Defines.cpp

# The actual cpp "executable". 
#   -E: make gcc just run the preprocessor
#   -x c: forces gcc to consider the input "C" code, despite the .Java suffix
#   -P:no explicit line/file info  (though this messes up the implicit line numbers!).
#   -Wcomment: warn about bad comments
#   -nostdinc: use none of the standard #include directories
#   -undef: no pre-defined macros 
#   -C: leave comments in preprocessed source
ifndef PREPROCESSOR
PREPROCESSOR:=gcc -E -x c -P -Wcomment -nostdinc -undef -C
endif

# Use different CPP flags based on the build
CPP_FLAGS_debug:=-DDEBUG -UEXPOSED -UNDEBUG -DTEST
CPP_FLAGS_optimized:=-UEXPOSED -UDEBUG -DNDEBUG
CPP_FLAGS_exposed:=-UNDEBUG -DDEBUG -DEXPOSED -DTEST
CPP_FLAGS_profiled:=-UEXPOSED -UDEBUG -DNDEBUG -DPROFILED

CPP_FLAGS_nodeos_pj:=-Dpj_NODEOS
CPP_FLAGS_nodeos_moab:=-Dmoab_NODEOS


CPP_FLAGS:=$(CPP_FLAGS_$(BUILD)) $(CPP_FLAGS_nodeos_$(NODEOS)) -I. $(addprefix -imacros ,$(CPP_GLOBAL_MACROS))
#   -I.: search the current directory for -imacros file
#   -imacros <file>: process <file> and keep macro/define info

### The whole point of this file:
PREPROCESS:=$(PREPROCESSOR) $(CPP_FLAGS)


### Sanity check rule in case something happens to the macros file
$(CPP_GLOBAL_MACROS): 
	@echo "The global macro file $@ doesn't exist.  Something is very broken."
	@$(FAILURE)

###
### The ubiquitous mkdebug target
###

mkdebug:: mkdebug_Preprocess.mk

mkdebug_Preprocess.mk::
	@echo " "
	@echo "**** Preprocess.mk settings:"
	@echo " "
	@echo "    CPP                 = $(CPP)"
	@echo "    CPP_GLOBAL_MACROS   = $(CPP_GLOBAL_MACROS)"
	@echo "    CPP_FLAGS           = $(CPP_FLAGS)"
	@echo "    PREPROCESS          = $(PREPROCESS)"

.PHONY: mkdebug_Preprocess.mk
