/*
 * NativeNodeOS.cpp
 *
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

/*
 * Set of mappings for "native" implementations.
 */

#define ID_JAVA_NODEOS 1
#define ID_C_NODEOS 2


#if defined(pj_NODEOS)
#define NATIVE_NODEOS ID_JAVA_NODEOS
#elif defined(moab_NODEOS)
#define NATIVE_NODEOS ID_C_NODEOS
#else
#error NativeNodeOS: Must pick nodeos impl (pj_NODEOS or moab_NODEOS)
#endif
/* -- */

#if NATIVE_NODEOS == ID_JAVA_NODEOS
#define NATIVE_BUFFERHANDLE		pj_BufferHandle
#define NATIVE_CREDENTIALS		pj_Credentials
#define NATIVE_CHAN_STATS		pj_ChanStats
#define NATIVE_INFO			pj_Info
#define NATIVE_FLOW			pj_Flow
#define NATIVE_FLOW_HANDLE		pj_FlowHandle
#define NATIVE_FLOW_STATS		pj_FlowStats
#define NATIVE_REMOTE_FLOW		pj_RemoteFlow
#define NATIVE_THREAD			pj_Thread
#define NATIVE_THREADPOOL		pj_ThreadPool
#define NATIVE_THREADPOOLSPEC		pj_ThreadPoolSpec
#define NATIVE_TIMESPEC			pj_TimeSpec
#define NATIVE_MEMSPEC			pj_MemSpec
#define NATIVE_INCHANNEL		pj_InChannel
#define NATIVE_OUTCHANNEL		pj_OutChannel
#define NATIVE_CUTTHROUGHCHANNEL	pj_CutThroughChannel
#define NATIVE_INCHANNELHANDLE		pj_InChannelHandle
#define NATIVE_DEMULTIPLEXKEY		pj_DemultiplexKey
#define NATIVE_DEBUG			pj_Debug
#define NATIVE_SETUP			pj_Setup
#define NATIVE_NODE			pj_Node

#elif NATIVE_NODEOS == ID_C_NODEOS
#define NATIVE_BUFFERHANDLE		moab_BufferHandle
#define NATIVE_CREDENTIALS		moab_Credentials
#define NATIVE_CHAN_STATS		moab_ChanStats
#define NATIVE_INFO			moab_Info
#define NATIVE_FLOW			moab_Flow
#define NATIVE_FLOW_HANDLE		moab_FlowHandle
#define NATIVE_FLOW_STATS		moab_FlowStats
#define NATIVE_REMOTE_FLOW		moab_RemoteFlow
#define NATIVE_THREAD			moab_Thread
#define NATIVE_THREADPOOL		moab_ThreadPool
#define NATIVE_THREADPOOLSPEC		moab_ThreadPoolSpec
#define NATIVE_TIMESPEC			moab_TimeSpec
#define NATIVE_MEMSPEC			moab_MemSpec
#define NATIVE_INCHANNEL		moab_InChannel
#define NATIVE_OUTCHANNEL		moab_OutChannel
#define NATIVE_CUTTHROUGHCHANNEL	moab_CutThroughChannel
#define NATIVE_INCHANNELHANDLE		moab_InChannelHandle
#define NATIVE_DEMULTIPLEXKEY		moab_DemultiplexKey
#define NATIVE_DEBUG			moab_Debug
#define NATIVE_SETUP			moab_Setup
#define NATIVE_NODE			moab_Node

#endif /* Picking a Native NodeOS */
