/*
 * PingFlow.Java
 *
 * Copyright (c) 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */
package JANOS_APPS_PKG.ping;

import JANOS_NODEOS_PKG.Flow;
import JANOS_NODEOS_PKG.Node;
import JANOS_NODEOS_PKG.FlowRun;
import JANOS_NODEOS_PKG.Address;
import JANOS_NODEOS_PKG.TimeSpec;
import JANOS_NODEOS_PKG.InChannel;
import JANOS_NODEOS_PKG.OutChannel;
import JANOS_NODEOS_PKG.DeliverFunc;
import JANOS_NODEOS_PKG.BufferHandle;
import JANOS_NODEOS_PKG.DemultiplexKey;
import JANOS_NODEOS_PKG.CommSpaceHandle;

import JANOS_PROTOCOLS_PKG.ip.IPHeader;
import JANOS_PROTOCOLS_PKG.icmp.ICMPEcho;
import JANOS_PROTOCOLS_PKG.icmp.ICMPHeader;

/**
 * A flow implementation of `ping'.
 */
public class PingFlow
	extends FlowRun
	implements DeliverFunc
{
	private PingArgs pa;
	private Flow flow;

	private InChannel ic;
	private OutChannel oc;

	protected void run(Flow flow,
			   String args[],
			   CommSpaceHandle commSpaceHandle)
	{
		try
		{
			DemultiplexKey dk;
			BufferHandle bh;
			int lpc, count;
			PingPacket pp;
			
			this.flow = flow;
			/* Get any arguments */
			this.pa = new PingArgs(args);
			/* Create our packet */
			pp = new PingPacket(this.pa.getSource(),
					    this.pa.getDestination(),
					    (short)0,
					    (short)0);
			/* Setup our InChannel to receive replies on... */
			dk = new DemultiplexKey(flow);
			pp.matchPingReply(dk);
			this.ic = InChannel.create(
				flow,
				dk,
				this.pa.getInputProtocolSpec(),
				this.pa.getInputAddressSpec(),
				this);
			this.ic.addBuffers(10);
			/* Setup the OutChannel to send the packets out */
			this.oc = OutChannel.create(
				flow,
				this.pa.getOutputProtocolSpec(),
				this.pa.getOutputAddressSpec());
			bh = BufferHandle.create(0, pp.getLength());
			count = this.pa.getCount();
			Node.println("PING ("
				     + Address.toString(this.pa.
							getDestination())
				     + "): "
				     + bh.getLength()
				     + " data bytes");
			for( lpc = 0; (count == -1) || (lpc < count); lpc++ )
			{
				pp.setSequenceNumber((short)lpc);
				pp.marshal(bh, 0);
				this.oc.send(bh);
				edu.utah.janos.nodeos.Thread.sleep(
					this.pa.getWaitTime() * 1000, 0);
			}
		}
		catch(Exception e)
		{
			Node.println("Caught exception: " + e);
			Node.printStackTrace(e);
		}
	}

	public void receive(BufferHandle bh, int payloadOffset)
	{
		try
		{
			PingPacket pp = new PingPacket();
			TimeSpec ts = new TimeSpec();
			IPHeader ip = pp.getIPHeader();
			ICMPHeader icmp = pp.getICMPHeader();
			double millis;

			pp.unmarshal(bh, 0);
			TimeSpec.getTime(ts);
			ts.subtract(pp.getTimeSegment().getTime());
			millis = ts.getTotalMillis();
			Node.println(""
				     + bh.getLength()
				     + " bytes from "
				     + Address.toString(ip.getSource())
				     + ": icmp_seq="
				     + pp.getSequenceNumber()
				     + " ttl="
				     + (ip.getTTL() & 0xff)
				     + " time="
				     + millis
				     + " ms");
		}
		catch(Exception e)
		{
		}
	}
}
