/*
 * PingPacket.Java
 *
 * Copyright (c) 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */
package JANOS_APPS_PKG.ping;

import JANOS_NODEOS_PKG.TimeSpec;
import JANOS_NODEOS_PKG.DemultiplexKey;

import JANOS_UTIL_PKG.PacketSegment;

import JANOS_PROTOCOLS_PKG.ip.IPHeader;
import JANOS_PROTOCOLS_PKG.icmp.ICMPEcho;
import JANOS_PROTOCOLS_PKG.icmp.ICMPHeader;

public class PingPacket
	extends ICMPEcho
{
	public static final int TIME_SLOT = ICMPEcho.ID_SEQ_SLOT + 1;
	
	public PingPacket(int src, int dst, short identifier, short seqNum)
	{
		super(src, dst, identifier, seqNum, new TimeSegment());
	}

	public PingPacket()
	{
		super(true, new TimeSegment());
	}

	public void matchPingReply(DemultiplexKey dk)
	{
		PacketSegment segs[] = this.getSegments();
		IPHeader iph = (IPHeader)segs[ICMPEcho.IP_SLOT];
		
		IPHeader.matchProtocol(dk, 0, IPHeader.PROTO_ICMP);
		IPHeader.matchSource(dk, 0, iph.getDestination());
		IPHeader.matchDestination(dk, 0, iph.getSource());
		ICMPHeader.matchType(dk, iph.getLength(),
				     ICMPHeader.ECHOREPLY);
		ICMPHeader.matchCode(dk, iph.getLength(), (byte)0);
	}

	public IPHeader getIPHeader()
	{
		return (IPHeader)this.getSegments()[ICMPEcho.IP_SLOT];
	}
	
	public ICMPHeader getICMPHeader()
	{
		return (ICMPHeader)this.getSegments()[ICMPEcho.ICMP_SLOT];
	}
	
	public TimeSegment getTimeSegment()
	{
		return (TimeSegment)this.getSegments()[TIME_SLOT];
	}
}
