/*
 * ClassInstantiationKey.Java
 *
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */
package JANOS_NODEOS_PKG;

/**
 * Blaouaufngg.  Janos uses reflection to instantiate the first object
 * in a flow, so that the object can't have pointers to objects in
 * other flows.  Unfortunately, the object's class usually belongs to
 * another package, so it must be public and have a public constructor
 * in order for JDK1.1-style reflection to work.  But this means anyone
 * can instantiate the class, which may not be desirable.  To prevent
 * this, Janos passes a ClassInstantiationKey into the constructor (if
 * there is a constructor taking one ClassInstantiationKey as an argument),
 * and the constructor verifies that the key matches its own class:
 *
 * public class Foo
 * {
 *     public Foo(ClassInstantiationKey key) throws SecurityException
 *     {
 *         key.checkEquals(Foo.class);
 *     }
 * }
 *
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu">Janos Ministry of Development</a>
 */
public final class ClassInstantiationKey
{
	private final Class c1;

	PACKAGE ClassInstantiationKey(Class c1)
	{
		this.c1 = c1;
	}

	public void checkEquals(Class c2)
		throws SecurityException
	{
		if (c1 != c2)
		{
			throw new SecurityException(c1 + " does not equal " + c2);
		}
	}
}
