/*
 * CommHashtable.java
 *
 * Copyright (c) 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package JANOS_NODEOS_PKG;

import java.util.Hashtable;

import JVM_SYS_PKG.Exportable;
import JVM_SYS_PKG.TemporaryExport;

/**
 * The CommHashtable wraps a regular Hashtable and makes it available for
 * export to other classes.
 */
PACKAGE class CommHashtable
	extends Exportable
	implements TemporaryExport
{
	/**
	 * Hashtable to store pairs of CommSpaceElements.
	 */
	private final Hashtable table = new Hashtable();

	/**
	 * Construct an empty CommHashtable.
	 */
	PACKAGE CommHashtable()
	{
	}

	/**
	 * @param id The identifier to be copied and stored in the hash table.
	 * @param value The value to be copied and stored in the hash table.
	 * @return Null or the previous value paired with the given identifier.
	 */
	PACKAGE CommSpaceElement put(CommSpaceElement id,
				     CommSpaceElement value)
	{
		CommSpaceElement retval;
		
		retval = (CommSpaceElement)this.table.
			put(CommSpaceElement.copy(id),
			    CommSpaceElement.copy(value));
		return retval;
	}

	/**
	 * @param id The identifier to lookup in the table.
	 * @return Null or the value paired with the given identifier.
	 */
	PACKAGE CommSpaceElement lookup(CommSpaceElement id)
	{
		return (CommSpaceElement)this.table.get(id);
	}

	/**
	 * @param id The identifier to remove from the hash table.
	 */
	PACKAGE void remove(CommSpaceElement id)
	{
		this.table.remove(id);
	}

	/**
	 * @param id The identifier to check for in the table.
	 * @return True if the identifier was found in the table, false
	 * otherwise.
	 */
	PACKAGE boolean containsKey(CommSpaceElement id)
	{
		return this.table.containsKey(id);
	}
}
