/*
 * CommHashtableHandle.Java
 *
 * Copyright (c) 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package JANOS_NODEOS_PKG;

import JVM_SYS_PKG.Importable;
import JVM_SYS_PKG.BooleanVisitor;
import JVM_SYS_PKG.CarrierVisitor;
import JVM_SYS_PKG.DeadTeamException;
import JVM_SYS_PKG.ThrowInVisitException;

/**
 * A handle for a CommHashtable that may be in another Flow.
 */
public class CommHashtableHandle
	extends Importable
{
	/**
	 * Construct a new CommHashtable and attach this handle to it.
	 */
	public CommHashtableHandle()
	{
		new CommHashtable().subscribe(this);
	}

	/**
	 * Copy constructor.
	 *
	 * @param chh The handle to copy.
	 */
	public CommHashtableHandle(CommHashtableHandle chh)
	{
		this.copy(chh);
	}

	/**
	 * @param ch The CommHashtable referred to by this handle.
	 * @param code The code to run in the flow that owns the hash table.
	 * @return Null or a copy of the CommSpaceElement left in the
	 * code.value field.
	 */
	private CommSpaceElement visitCommHashtable(final CommHashtable ch,
						    final CarrierVisitor code)
	{
		CommSpaceElement retval = null;
		
		try
		{
			ch.pushOwnerActivation().visit(code);
			if( code.value != null )
			{
				CommSpaceElement cse = (CommSpaceElement)code.
					value;

				retval = (CommSpaceElement)cse.clone();
			}
		}
		catch(DeadTeamException dte)
		{
		}
		catch(ThrowInVisitException eiv)
		{
			/* Should we do something here? */
		}
		ch.popOwnerActivation();
		return retval;
	}

	/**
	 * @param id Identifier to be paired with a value
	 * @param val Value to pair with an identifier
	 * @return The element that was previously paired with the identifier.
	 */
	public CommSpaceElement put(final CommSpaceElement id,
				    final CommSpaceElement val)
	{
		final CommHashtable ch = (CommHashtable)this.get();
		
		return this.visitCommHashtable(ch, new CarrierVisitor() {
			public void run()
			{
				this.value = ch.put(id, val);
			}
		});
	}

	/**
	 * @param id The identifier to lookup in the hash table.
	 * @return Null or the element that is paired with the given identifier
	 */
	public CommSpaceElement lookup(final CommSpaceElement id)
	{
		final CommHashtable ch = (CommHashtable)this.get();
		
		return this.visitCommHashtable(ch, new CarrierVisitor() {
			public void run()
			{
				this.value = ch.lookup(id);
			}
		});
	}

	/**
	 * @param id The identifier to remove from the hash table.
	 */
	public void remove(final CommSpaceElement id)
	{
		final CommHashtable ch = (CommHashtable)this.get();

		this.visitCommHashtable(ch, new CarrierVisitor() {
			public void run()
			{
				ch.remove(id);
			}
		});
	}

	/**
	 * @param id The identifier to check for in the hash table.
	 * @return True if the identifier was found in the table, false
	 * otherwise.
	 */
	public boolean containsKey(final CommSpaceElement id)
	{
		final CommHashtable ch = (CommHashtable)this.get();
		boolean retval = false;
		
		try
		{
			BooleanVisitor bv;
			
			ch.visitOwner(bv = new BooleanVisitor() {
				public void run()
				{
					this.value = ch.containsKey(id);
				}
			});
			retval = bv.value;
		}
		catch(DeadTeamException dte)
		{
		}
		catch(ThrowInVisitException tive)
		{
		}
		return retval;
	}
}
