/*
 * CommQueue.Java
 *
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */
package JANOS_NODEOS_PKG;

import java.util.Vector; // XXX

import JVM_SYS_PKG.Exportable;
import JVM_SYS_PKG.TemporaryExport;

/**
 * A shareable FIFO queue of CommSpaceElements.
 *
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu">Janos Ministry of Development</a>
 */
PACKAGE final class CommQueue
	extends Exportable
	implements TemporaryExport
{
	// Sorry about the inefficient implementation:
	private final Vector elements = new Vector();

	/**
	 * @param elem The element to copy and enqueue.
	 */
	public void enqueue(CommSpaceElement elem)
	{
		synchronized(this.elements)
		{
			this.elements.addElement(elem.clone());
			this.elements.notify();
		}
	}

	/**
	 * @param elems The array of elements to copy and atomically add to the
	 * queue.
	 */
	public void enqueue(CommSpaceElement elems[])
	{
		synchronized(this.elements)
		{
			int lpc;

			for( lpc = 0; lpc < elems.length; lpc++ )
			{
				this.elements.addElement(elems[lpc].clone());
			}
			this.elements.notify();
		}
	}

	/**
	 * Wait to take an element from the queue.
	 * <p>
	 * May throw InterruptedException if the wait() is
	 * interrupted.  The caller will have to restart the
	 * waitToTake().
	 *
	 * @return The next element on the queue.
	 */
	public CommSpaceElement waitToTake()
		throws InterruptedException
	{
		synchronized(elements)
		{
			if (elements.size() == 0)
				elements.wait();
			CommSpaceElement elem = (CommSpaceElement)
				elements.firstElement();
			elements.removeElementAt(0);

			return elem;
		}
	}
}
