/*
 * CommQueuePuller.Java
 *
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */
package JANOS_NODEOS_PKG;

import JVM_SYS_PKG.Importable;

/**
 * A handle for the owner of the CommQueue to pull elements out of it.
 */
public final class CommQueuePuller
	extends Importable
{

	/**
	 * Construct CommQueuePuller handle and a CommQueue for it to pull
	 * CommSpaceElements from.
	 */
	public CommQueuePuller()
	{
		new CommQueue().subscribe(this);
	}

	/**
	 * Take an element from the queue, or wait for one to be added and
	 * return that.
	 *
	 * @return The next element on the queue.
	 */
	public CommSpaceElement waitToTake()
		throws InterruptedException
	{
		CommSpaceElement retval = null;
		CommQueue cq;

		if( (cq = (CommQueue)this.get()) != null )
		{
			retval = cq.waitToTake();
		}
		return retval;
	}
	
}
