/*
 * CommQueuePusher.Java
 *
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package JANOS_NODEOS_PKG;

import JVM_SYS_PKG.Visitor;
import JVM_SYS_PKG.Importable;
import JVM_SYS_PKG.DeadTeamException;
import JVM_SYS_PKG.ThrowInVisitException;

/**
 * A handle for other flows to push elements onto a CommQueue.
 */
public final class CommQueuePusher
	extends Importable
{

	/**
	 * Construct an empty pusher, it can be bound to a queue later.
	 */
	public CommQueuePusher()
	{
	}

	/**
	 * Construct a handle that will push elements onto the queue managed
	 * by the puller.
	 *
	 * @param puller The handle for the CommQueue we want to push elements
	 * onto.
	 */
	public CommQueuePusher(CommQueuePuller puller)
	{
		this.copy(puller);
	}

	/**
	 * @param pusher The handle for the CommQueue we want to push elements
	 * onto.
	 */
	public CommQueuePusher(CommQueuePusher pusher)
	{
		this.copy(pusher);
	}

	/**
	 * Bind this pusher to the CommQueue referred to by the argument.
	 *
	 * @param pusher The handle to the CommQueue we should bind ourselves
	 * too.
	 */
	public void bind(CommQueuePusher pusher)
	{
		this.copy(pusher);
	}

	/**
	 * @param cq The CommQueue referred to by this handle.
	 * @param code The code to run in the flow that owns the hash table.
	 */
	private void visitCommQueue(final CommQueue cq, final Visitor code)
	{
		try
		{
			if( cq != null )
				cq.visitOwner(code);
		}
		catch(DeadTeamException dte)
		{
		}
		catch(ThrowInVisitException eiv)
		{
		}
	}
	
	/**
	 * @param cse The element to enqueue.
	 */
	public void enqueue(final CommSpaceElement cse)
	{
		final CommQueue cq = (CommQueue)this.get();

		this.visitCommQueue(cq, new Visitor() {
			public void run()
			{
				cq.enqueue(cse);
			}
		});
	}

	/**
	 * @param cse The array of elements to atomically add to the queue.
	 */
	public void enqueue(final CommSpaceElement cse[])
	{
		final CommQueue cq = (CommQueue)this.get();

		this.visitCommQueue(cq, new Visitor() {
			public void run()
			{
				cq.enqueue(cse);
			}
		});
	}
}
