/*
 * CutThroughChannel.Java
 *
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */
package JANOS_NODEOS_PKG;

import java.io.IOException; // XXX

/**
 * Java wrapper for a CutThroughChannel.
 * 
 * @see InChannel
 * @see OutChannel
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu">Janos Ministry of Development</a>
 */
public final class CutThroughChannel
	extends NATIVE_CUTTHROUGHCHANNEL
{
	CHECK_NATIVE_NODEOS

	public static CutThroughChannel
	create(Flow flow,
	       DemultiplexKey classifier,
	       ThreadPoolSpec inchan_tps,
	       NetworkSpec outchan_ns,
	       String inProtocolSpec,
	       String outProtocolSpec,
	       String inAddressSpec,
	       String outAddressSpec,
	       Address destAddr)
		throws IOException, // XXX
		PacketAddressFormatException,
		ProtocolSpecFormatException
	{
		// XXX this isn't sufficient protection for the rest of the kernel..
		if((flow == null)
		   || (classifier == null)
		   || (inProtocolSpec == null)
		   || (outProtocolSpec == null)
		   || (inAddressSpec == null)
		   || (outAddressSpec == null)
		   || (destAddr == null))
			throw new NullPointerException();

		return new CutThroughChannel(flow, classifier,
					     inchan_tps, outchan_ns,
					     inProtocolSpec,
					     outProtocolSpec,
					     inAddressSpec,
					     outAddressSpec,
					     destAddr);
	}

	/**
	 * Constructors are private so I can do a validity check on the
	 * arguments.
	 */
	private CutThroughChannel(Flow flow,
				  DemultiplexKey classifier,
				  ThreadPoolSpec inchan_tps,
				  NetworkSpec outchan_ns,
				  String inProtocolSpec,
				  String outProtocolSpec,
				  String inAddressSpec,
				  String outAddressSpec,
				  Address destAddr)
		throws IOException, // XXX
		PacketAddressFormatException,
		ProtocolSpecFormatException
	{
		super(flow.getNativeFlow(),
		      classifier.getNativeKey(),
		      inchan_tps.getNativeSpec(),
		      outchan_ns,
		      inProtocolSpec,
		      outProtocolSpec,
		      inAddressSpec,
		      outAddressSpec,
		      destAddr);
	}
	
	/**
	 * Add a buffer to the channel
	 */
	public void addBuffer(BufferHandle bufh)
	{
		NATIVE_BUFFERHANDLE nb;

		require(bufh); // XXX 

		nb = bufh.getNativeBufH();
		super.native_addBuffer(nb);
	}

	/**
	 * Remove a buffer from the channel. 
	 * A new BufferHandle is allocated to point to the buffer.
	 * <p>
	 * XXX new a call that takes a BufferHandle and points it at
	 * the underlying buffer object.
	 *
	 * @allocation Allocates a new BufferHandle()
	 * @return A new BufferHandle pointing to the removed buffer.
	 */
	public BufferHandle remBuffer()
	{
		NATIVE_BUFFERHANDLE nbh;
		nbh = super.native_remBuffer();
		assert(nbh instanceof BufferHandle);
		return (BufferHandle)nbh;
	}
	
	PACKAGE void destroy()
	{
		TRACE(Trace.NODEOS_DESTROY, "Destroying " + this);
		super.destroy();
	}

	public String toString()
	{
		return super.toString();
	}
}


