/*
 * FlowStartSpec.Java
 *
 * Copyright (c) 1999, 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */
package JANOS_NODEOS_PKG;

/**
 * Various attributes of a flow that must be defined before a
 * Flow can be created.
 *
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu">Janos Ministry of Development</a>
 */
public class FlowStartSpec
	implements Cloneable
{
	/**
	 * The `main' class to start up in the flow.
	 */
	private String startClass;

	/**
	 * The arguments to pass to the `startClass' when starting it.
	 */
	private String[] args;

	/**
	 * CommSpaceHandle used to connect to another flow.
	 */
	private CommSpaceHandle commSpaceHandle;

	public FlowStartSpec(String startClass,
			     String[] args,
			     CommSpaceHandle commSpaceHandle)
	{
		require(startClass);
		require(args);
		// XXX don't require(commSpaceHandle);

		this.startClass = startClass;
		this.args = args;
		this.commSpaceHandle = commSpaceHandle;
	}

	public String getStartClass()
	{
		return this.startClass;
	}

	public String[] getArguments()
	{
		return this.args;
	}

	public CommSpaceHandle getCommSpaceHandle()
	{
		return this.commSpaceHandle;
	}

	public Object clone()
	{
		String argsClone[] = new String[this.args.length];
		CommSpaceHandle csh = null;
		int lpc;

		for( lpc = 0; lpc < argsClone.length; lpc++ )
		{
			argsClone[lpc] = CloneFactory.clone(this.args[lpc]);
		}
		if( this.commSpaceHandle != null )
			csh = (CommSpaceHandle)this.commSpaceHandle.clone();
		return new FlowStartSpec(CloneFactory.clone(this.startClass),
					 argsClone,
					 csh);
	}
	
	public String toString()
	{
		String retval;
		int lpc;

		require(this.startClass);
		require(this.args);
		
		retval = "FlowStartSpec[startClass="
			+ this.startClass
			+ "; args={ ";
		for( lpc = 0; lpc < this.args.length; lpc++ )
		{
			if( lpc > 0 )
				retval += ", ";
			retval += this.args[lpc];
		}
		retval += " };";

		if (commSpaceHandle != null)
			retval += " (has csh)";
		else
			retval += " (no csh)";

		retval += "]";
		return retval;
	}
}
