/*
 * FlowStats.Java
 *
 * Copyright (c) 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */
package JANOS_NODEOS_PKG;

/**
 * FlowStats is used to store statistics pertaining to a flow in the system.
 *
 * XXX This interface is pretty terrible, we should just have a few functions
 * for each return type (int/long/float) and then pass in a constant
 * indicating which stat we're intereseted in.
 */
public final class FlowStats
	extends NATIVE_FLOW_STATS
{
	public int getThreadCount()
	{
		return super.getThreadCount();
	}

	public int getMaxThreads()
	{
		return super.getMaxThreads();
	}

	public int getActivationCount()
	{
		return super.getActivationCount();
	}

	public int getCPUUsage()
	{
		return super.getCPUUsage();
	}

	public int getMemoryUsage()
	{
		return super.getMemoryUsage();
	}

	/**
	 * @return The length, in bytes, of the space required
	 * to store a linearized version of this stats
	 * buffer.
	 */
	public int getLength()
	{
		return super.getLength();
	}
	
	/**
	 * @param bh The BufferHandle to marshal the stats into.
	 * @param offset The offset into the BufferHandle where the stats
	 *   should be marshalled.
	 * @return The number of bytes written.
	 */
	public int marshal(BufferHandle bh, int offset)
	{
		return super.marshal(bh.getNativeBufH(), offset);
	}
	
	/**
	 * @param bh The BufferHandle to unmarshal the stats from.
	 * @param offset The position in the buffer where the marshalled stats
	 *   are located.
	 * @return The number of bytes consumed.
	 */
	public int unmarshal(BufferHandle bh, int offset)
	{
		return super.unmarshal(bh.getNativeBufH(), offset);
	}
	
	PACKAGE NATIVE_FLOW_STATS getNative()
	{
		return this;
	}

	public String toString()
	{
		return "FlowStats[threadCount="
			+ this.getThreadCount()
			+ "; maxThreads="
			+ this.getMaxThreads()
			+ "; activationCount="
			+ this.getActivationCount()
			+ "; cpuUsage="
			+ this.getCPUUsage()
			+ "; memUsage="
			+ this.getMemoryUsage()
			+ "]";
	}
}
