/*
 * InChannelHandle.Java
 *
 * Copyright (c) 1999, 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */
package JANOS_NODEOS_PKG;


/**
 * A cross-process handle on a Java NodeOS InChannel.
 * <p>
 * Packets can be inserted into an InChannel via an InChannelHandle.
 *
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu">Janos Ministry of Development</a>
 * @see InChannel
 */
public final class InChannelHandle
	extends NATIVE_INCHANNELHANDLE
{
	CHECK_NATIVE_NODEOS

	/**
	 * Construct a handle that doesn't refer to anything.
	 */
	public InChannelHandle()
	{
		super();
		super.setChannel(null);
	}

	/**
	 * @param inChan The InChannel object that this handle should refer
	 *   too.
	 */
	public InChannelHandle(InChannel inChan)
	{
		super();

		require(inChan); // XXX

		super.setChannel(inChan.getNativeInChan());
	}

	/**
	 * @param otherChan The InChannelHandle to copy.
	 */
	public void copy(InChannelHandle otherChan)
	{
		require(otherChan); // XXX

		super.copy((NATIVE_INCHANNELHANDLE)otherChan);
	}

	/**
	 * Push a buffer onto the associated InChannel.  The given
	 * packet data is either copied into or swapped with 
	 * packet data on the InChannel.
	 * <p>
	 * Returns true if the packet was swapped/copied.  False if the
	 * not (the channel is dead, there was no buffer space, etc).
	 */
	public boolean handoff(BufferHandle pkt)
	{
		boolean rv;
		pkt.systemOwns(); // XXX 

		// XXX need to check that this ich points to an ic.

		// XXX why synchro?
		synchronized(this)
		{
			NATIVE_BUFFERHANDLE nb;
			nb = pkt.getNativeBufH();
			rv = super.handoff(pkt.getNativeBufH());
		}

		pkt.userOwns();
		return rv;
	}

	public boolean handoffv(BufferHandle[] bhl)
	{
		// XXX bhl[0..].systemOwns();
		synchronized(this)
		{
			super.handoffv(bhl);
		}
		// XXX bhl[0..].userOwns();

		return false;
	}

	public String toString()
	{
		return super.toString();
	}
}

