/*
 * Status.Java
 *
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */
package JANOS_NODEOS_PKG;

import java.util.Hashtable; // XXX

/**
 * Obtain status information on the state of a node.
 * <p>
 * This is the public interface to querying stats on the
 * node.
 * <p>
 * XXX ideally, most of this status stuff should be external
 * to the nodeos core package.
 *
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu">Janos Ministry of Development</a>
 */
public class Status
{
	private static final Status statusObj = new Status();

	/**
	 * A `root' InfoHook that is used to track the rest of them
	 */
	private static final InfoHook rootHook = new InfoHook("root");

	/**
	 * Private constructor.  Only one status object is ever
	 * created: the one referenced in the statusObj field.
	 */
	private Status()
	{
	}

	private static boolean gaveStat = false;

	/**
	 * Return the single global status object exactly once.
	 * First come, first served.  The caller can decide what
	 * sort of access controls to implement.
	 * <p>
	 * XXX drop lame access controls.  Let EE do all the controls.
	 */
	public static Status getStatusObject()
	{
		if (gaveStat)
			return null;
		gaveStat = true;
		return statusObj;
	}

	/**
	 * Add an InfoHook that can be inspected by the Status user
	 */
	public static void addInfoHook(InfoHook ih)
	{
		rootHook.addChild(ih);
	}

	/**
	 * Get the information string corresponding to the given module name
	 *
	 * @param name The name of the InfoHook to lookup
	 */
	public String[] getInfo(String name)
	{
		String retval[] = null;
		InfoHook child;

		child = rootHook.findChild(name);
		if( child != null )
		{
			retval = child.getInfo();
		}
		return retval;
	}

	/**
	 * Get the list of InfoHook's available
	 *
	 * @return An array of strings containing the names of the info hooks
	 */
	public String[] getInfoList()
	{
		return rootHook.getChildList();
	}
	
}
