/*
 * Thread.Java
 *
 * Copyright (c) 1999, 2000, 2001 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */
package JANOS_NODEOS_PKG;

/**
 * A Janos Thread. <code>java.lang.Thread</code> is not visible to
 * threads running on the NodeOS.
 * <p>
 * There are no visible Thread objects in Janos (all the methods
 * here are static ones).  New threads are created either implicitly
 * in response to packets, or explicitly by a call to
 * <code>Flow.threadStart(...)</code>.
 *
 * @see Flow#threadStart(java.lang.Runnable,String)
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu">Janos Ministry of Development</a>
 */
public class Thread
{
	CHECK_NATIVE_NODEOS

	private Thread()
	{
		// Cannot create a Java Thread object
	}

	/**
	 * The mighty directed yield.
	 * <p>
	 * The scheduler may honor the argument.  Or it may
	 * laugh at your suggestion and put the target thread
	 * in a black hole just to spite you.
	 * <p>
	 * Operates on the current thread, implicitly.
	 *
	 * @param yieldTo the thread that should be run in deference to the current thread.
	 */
	public static void yieldTo(Thread yieldTo)
	{
		//NATIVE_THREAD.yieldTo((yieldTo == null)?(null):(yieldTo.nThread));
		NATIVE_THREAD.yieldTo(null);
	}
	
	/**
	 * An undirected yield.
	 * <p>
	 * Equivalent to <code>yield(null)</code>.
	 */
	public static void yield()
	{
		yieldTo(null);
	}

	/**
	 * Put the current thread to sleep for at least
	 * the given amount of time.
	 * <p>
	 * Operates on the current thread, implicitly.
	 */
	public static void sleep(long millis, int nanos)
		throws InterruptedException
	{
		NATIVE_THREAD.sleep(millis, nanos);
	}

	/**
	 * Set the per-thread object pointer.
	 * Operates on the current thread, implicitly.
	 * <p>
	 * Returns the old object.
	 */
	public static Object setObject(Object obj)
	{
		return NATIVE_THREAD.setObject(obj);
	}

	/**
	 * Get the per-thread object pointer.
	 * Operates on the current thread, implicitly.
	 */
	public static Object getObject()
	{
		return NATIVE_THREAD.getObject();
	}
}
