/*
 * moab_BufferHandle.Java
 *
 * Copyright (c) 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */
package JANOS_NODEOS_PKG;

/**
 * Look at the Pure Java Buffer Handle (pj_BufferHandle) for
 * documentation on individual methods.
 *
 * @see BufferHandle
 * @see pj_BufferHandle
 */
PACKAGE class moab_BufferHandle
{
	PACKAGE static native int getMaxPacketLen();

	protected moab_BufferHandle(moab_BufferHandle other, int offset, int length)
	{
		init(other, offset, length);
	}

	private native void init(moab_BufferHandle other, int offset, int length);
	
	PACKAGE native int getLength();

	PACKAGE native int getOffset();

	PACKAGE native void reset();

	PACKAGE native void newBuffer(int offset, int length);

	PACKAGE native void cloneHandle(moab_BufferHandle src, int fromOffset, int fromLength);

	PACKAGE native void swapWith(moab_BufferHandle src, int fromOffset, int fromLength);

	PACKAGE native byte getByte(int index);

	PACKAGE native void setByte(int index, byte val);

	PACKAGE native short getShort(int index);

	PACKAGE native void setShort(int index, short val);

	PACKAGE native int getInt(int index);

	PACKAGE native void setInt(int index, int val);

	PACKAGE native void getBytes(int srcPosition, byte[] dest,
				     int destPosition, int destLength);
	
	PACKAGE native void setBytes(int destPosition, byte[] src,
				     int srcPosition, int srcLength);

	PACKAGE native void getInts(int srcPosition, int[] dest,
				    int destPosition, int destLength);

	PACKAGE native void setInts(int destPosition, int[] src,
				    int srcPosition, int srcLength);

	PACKAGE native short computeChecksum(int start, int count);

	PACKAGE native boolean compareBytes(int offset, byte[] sequence, int seqOffset, int seqLen);
	
	PACKAGE native void destroy();

	protected void finalize()
	{
		this.destroy();
	}

	/**
	 * Write info about this packet buffer into the given char array.
	 * Return the number of char array entries used to describe the buffer.
	 */
	PACKAGE native int dbgPacketSummary(char[] sbuf, int offset, int len);
	
	public String toString()
	{
		return "moab_BufferHandle["
			+ "; off=" + this.getOffset()
			+ "; len=" + this.getLength()
			+ "]";
	}
}
