/*
 * moab_Debug.Java
 *
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */
package JANOS_NODEOS_PKG;

/**
 * see pj_Debug for doc.
 * <p>
 * XXX This class isn't acutally native yet.  Needs to switch
 * entirely over to native methods.
 * 
 * @see Debug
 * @see pj_Debug
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu">Janos Ministry of Development</a>
 */
public class moab_Debug
{
	CHECK_NATIVE_NODEOS

#ifdef DEBUG
	/**
	 * A native print routine that doesn't require the basic Java classes.
	 * Prints a trailing newline.
	 *
	 * @param str The string to print
	 */
	public static void println(String str)
	{
		java.lang.System.err.println(str);
	}
	

	/**
	 * A native print routine that doesn't require the basic Java classes.
	 * Does not print a trailing newline.
	 *
	 * @param str The string to print
	 */
	public static void print(String str)
	{
		java.lang.System.err.print(str);
	}
	
	/**
	 * A native print routine that doesn't require the basic Java classes.
	 * Prints the given integer as a string (in base 10).
	 *
	 * @param i the integer to print
	 */
	public static void printInt(int i)
	{
		java.lang.System.err.print(i);
	}

	/**
	 * A native print routine that doesn't require the basic Java classes.
	 * Prints the given integer as a string (in hex).
	 *
	 * @param i the integer to print
	 */
	public static void printIntHex(int i)
	{
		// XXX not implemented
		java.lang.System.err.print("0xXX");
	}

	/**
	 * A native print routine that doesn't require the basic Java classes.
	 * Prints the given char array as a sequence of characters.
	 *
	 * @param array the char array to print
	 * @param start the first element of the char array to print
	 * @param end the last element of the char array to print
	 */
	public static void printCharArray(char[] array, int start, int end)
	{
		// XXX not implemented
		print("char[] XX");
	}

	/**
	 * Print the stack trace of the given Throwable
	 * thread.
	 */
	public static void printStackTrace(Throwable t)
	{
		beginDebugOutput();
		t.printStackTrace(java.lang.System.err);
		endDebugOutput();
	}

	public static void exit()
	{
		Debug.println("Debug.exit invoked.");
		System.exit(1);
	}

	public static boolean checkLocked(Object obj)
	{
		// XXX cannot check in Pure Java
		return true;
	}

	/*
	 * Synchronize access to the debugging dump via
	 * a home-brewed binary semaphore.
	 */
	
	private final static Object lock = new Object();
	private static int count = 0;

	public static void beginDebugOutput()
	{
		synchronized(lock)
		{
			while (count != 0)
			{
				try
				{
					lock.wait();
				}
				catch (InterruptedException _)
				{
				}
			}
			count = 1;
		}
	}
	
	public static void endDebugOutput()
	{
		synchronized(lock)
		{
			count = 0;
			lock.notify();
		}
	}

#endif /*DEBUG*/
}
