/*
 * moab_FlowHandle.Java
 *
 * Copyright (c) 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */
package JANOS_NODEOS_PKG;

import JVM_SYS_PKG.Importable;

/**
 * See the implementation-independent FlowHandle for a description.
 *
 * @see FlowHandle
 */
PACKAGE class moab_FlowHandle
	extends Importable
{
	/**
	 * Create a new flow handle that points at the
	 * given flow.
	 */
	PACKAGE moab_FlowHandle(Flow f)
	{
		f.subscribe(this);
	}

	PACKAGE synchronized void getStats(moab_FlowStats fs)
	{
		Flow f = (Flow)this.get();

		f.getStats(fs);
	}

	PACKAGE synchronized boolean getInChannelStats(int index,
						       moab_ChanStats cs)
	{
		Flow f = (Flow)this.get();
		boolean retval = false;

		if( f != null )
		{
			moab_Flow mf = f.getNativeFlow();

			try
			{
				InChannel ic;
				
				ic = mf.getInChannel(index);
				ic.getStats(cs);
				retval = true;
			}
			catch(ArrayIndexOutOfBoundsException e)
			{
			}
		}
		return retval;
	}

	PACKAGE synchronized boolean getOutChannelStats(int index,
							moab_ChanStats cs)
	{
		Flow f = (Flow)this.get();
		boolean retval = false;

		if( f != null )
		{
			moab_Flow mf = f.getNativeFlow();

			try
			{
				OutChannel oc;
				
				oc = mf.getOutChannel(index);
				oc.getStats(cs);
				retval = true;
			}
			catch(ArrayIndexOutOfBoundsException e)
			{
			}
		}
		return retval;
	}

	PACKAGE void destroy()
	{
		Flow childFlow = (Flow)(super.get());

		/* XXX sanity check childFlow pointer? */

		/* XXX is this correct? */
		childFlow.destroy();
	}

#ifdef DEBUG
	PACKAGE String getName()
	{
		// XXX do I need to visit the owner or something to get the string? 

		Flow f = (Flow)(super.get());
		return f.getName();
	}
#endif
}
