/*
 * moab_OutChannel.Java
 *
 * Copyright (c) 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */
package JANOS_NODEOS_PKG;

import java.io.IOException;

IMPORT_DEBUG

/**
 * Wrapper for Moab OutChannel
 *
 * @see CutThroughChannel
 * @see InChannel
 * @see OutChannel
 */
PACKAGE class moab_OutChannel
{
	CHECK_NATIVE_NODEOS

	PACKAGE moab_OutChannel(moab_Flow flow,
				NetworkSpec ns,
				String protocolSpec,
				String addressSpec)
		throws IOException,
		       PacketAddressFormatException,
		       ProtocolSpecFormatException
	{
		moab_ChannelAddressSpec cas;
		moab_TransportInterface ti;
		String topProtocol;
		int bw, rc;

		// XXX the "socket:" thing is a hack for Moab
		if (protocolSpec.equals("if/ipv4/udp"))
		{
			protocolSpec = "socket:" + protocolSpec;
		}
		/*
		 * Get the top protocol that the address spec should
		 * correspond too.
		 */
		topProtocol = ProtocolSpec.lastComponent(protocolSpec);
		if( (ti = moab_Info.getTransports().
		     findInterface(topProtocol)) != null )
		{
			/*
			 * Found the interface for the top protocol, now create
			 * and initialize an address spec that moab can use.
			 */
			cas = (moab_ChannelAddressSpec)
				ti.createAddress(addressSpec);
		}
		else
		{
			String msg =
				"Protocol spec '"
				+ protocolSpec
				+ "' is not supported";
			
			throw new ProtocolSpecFormatException(msg);
		}
		bw = ns.getShare(ti.getBandwidth());
		/* Initialize the native data. */
		rc = init(flow,
			  bw,
			  protocolSpec,
			  cas);
		assert(rc == 0); // XXX
		flow.addChannel(this);
	}

	private native int init(moab_Flow flow,
				int bandwidth,
				String protocolSpec,
				moab_ChannelAddressSpec cas);

	PACKAGE native boolean send(moab_BufferHandle bh);

	PACKAGE native void sendAndRecycle(moab_BufferHandle bh, moab_InChannel inChan);
	
	PACKAGE native void sendAsynch(moab_BufferHandle bh, DeliverFunc callback);

	PACKAGE native void sendv(moab_BufferHandle[] bhl);
	
	PACKAGE native void getStats(moab_ChanStats cs);

	PACKAGE native void destroy();

	public String toString()
	{
		return "moab_OutChannel[...]";
	}
}
