/*
 * pj_PacketHandlerList.Java
 *
 * Copyright (c) 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */
package JANOS_NODEOS_PKG;

import java.util.Vector;
import java.net.DatagramPacket;
import java.net.Socket;

/**
 * pj_PortHandleList is a helper class for protocol layers that is
 * used to map a number of PortAddress-based PacketHandlers to an
 * individual socket.
 *
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu">Janos Ministry of Development</a>
 */
PACKAGE class pj_PacketHandlerList
{
	private final Vector masks = new Vector();
	
	/**
	 * Find a packet handler that matches the given socket, we
	 * just check the source masks of each handler.
	 */
	PACKAGE pj_ProtocolLayer.PacketHandler matchMask(Socket socket)
	{
		pj_ProtocolLayer.PacketHandler ph, retval = null;
		int size = this.masks.size();
		int lpc;
		
		for ( lpc = 0; (lpc < size) && (retval == null); lpc++ )
		{
			ph = (pj_ProtocolLayer.PacketHandler)this.masks.
				elementAt(lpc);
			if ( ((pj_PortAddressMask)ph.source).matchSocket(socket) )
				retval = ph;
		}
		return retval;
	}

	/**
	 * Find a packet handler that matches the given packet, we
	 * just check the source masks of each handler.
	 */
	PACKAGE pj_ProtocolLayer.PacketHandler matchMask(DatagramPacket dp)
	{
		pj_ProtocolLayer.PacketHandler ph, retval = null;
		int size = this.masks.size();
		int lpc;
		
		for ( lpc = 0; (lpc < size) && (retval == null); lpc++ )
		{
			ph = (pj_ProtocolLayer.PacketHandler)this.masks.
				elementAt(lpc);
			if ( ((pj_PortAddressMask)ph.source).matchPacket(dp) )
				retval = ph;
		}
		return retval;
	}

	/**
	 * Add a packet handler to this socket.
	 */
	PACKAGE void addHandler(pj_ProtocolLayer.PacketHandler ph)
	{
		int lpc, size;
		
		synchronized (this.masks)
		{
			size = this.masks.size();
			for ( lpc = 0; lpc < size; lpc++ )
			{
				pj_ProtocolLayer.PacketHandler elem =
					(pj_ProtocolLayer.PacketHandler)this.
					masks.elementAt(lpc);
				
				if ( ((pj_PortAddressMask)ph.source).priority() >=
				     ((pj_PortAddressMask)elem.source).priority() )
					break;
			}
			this.masks.insertElementAt(ph, lpc);
		}
	}
	
	/**
	 * Remove a packet handler from this socket.
	 */
	PACKAGE boolean remHandler(pj_ProtocolLayer.PacketHandler ph)
	{
		boolean retval = false;
		
		synchronized (this.masks)
		{
			this.masks.removeElement(ph);
			if ( this.masks.size() == 0 )
			{
				retval = true;
			}
		}
		return( retval );
	}
}
