/*
 * ICMPSummarySegment.Java
 *
 * Copyright (c) 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */
package JANOS_PROTOCOLS_PKG.icmp;

import JANOS_NODEOS_PKG.BufferHandle;

import JANOS_PROTOCOLS_PKG.ip.IPHeader;

import JANOS_UTIL_PKG.PacketSegment;
import JANOS_UTIL_PKG.PlainPacketSegment;

/**
 * The ICMPSummarySegment is a PacketSegment that encapsulates a summary of
 * an IP packet that was dropped/invalid/etc...
 */
PACKAGE class ICMPSummarySegment
	extends PlainPacketSegment
{
	/**
	 * Summarize the given IP packet.
	 *
	 * @param bh The buffer where the packet is stored.
	 * @param offset The offset into the buffer where the IP header starts.
	 * @param iplen The length of the IP header.
	 */
	private static byte[] summarize(BufferHandle bh, int offset, int iplen)
	{
		byte retval[] = new byte[(4		// Unused
					  + iplen	// IP header
					  + (64 / 8))];	// First 64 bits of the
							//   datagram

		bh.getBytes(offset, retval, 4, retval.length - 4);
		return retval;
	}
	
	/**
	 * Construct an ICMPSummarySegment from the given IP packet.
	 *
	 * @param bh The buffer where the packet is stored.
	 * @param offset The offset into the buffer where the IP header starts.
	 * @param iplen The length of the IP header.
	 */
	PACKAGE ICMPSummarySegment(BufferHandle bh, int offset, int iplen)
	{
		super(summarize(bh, offset, iplen));
	}
}
