/*
 * WriteableRouteTable.Java
 *
 * Copyright (c) 1999 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */
package JANOS_ROUTE_PKG;

import JANOS_NODEOS_PKG.NoSuchNodeAddress;
import JANOS_NODEOS_PKG.AddressFormatException;

import java.util.Enumeration; // XXX

/**
 * Interface for an updatable routing table.  The regular
 * route table is read-only.  Access to this class is controlled
 * (or, more acurately, supposed to be controlled) by controls on
 * a flow's namespace.
 * <p>
 * NOTE: If you add/change any interface in here, be sure to
 * update the ANTSR DynamicRouteTable as it has to hide all of
 * these interfaces.
 *
 * @see RouteTable
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu">Janos Ministry of Development</a>
 */
public class WriteableRouteTable
	extends RouteTable
{
	/**
	 * Create a WriteableRouteTable.
	 */
	public WriteableRouteTable() 
	{
		super();
	}

	/**
	 * Get the underlying read-only table.
	 */
	public RouteTable getRouteTable()
	{
		return this;
	}

	public Enumeration getRoutes()
	{
		return super.getRoutes();
	}

	/**
	 * Retrive the current route to get to the given active address.
	 */
	public RouteEntry getRoute(int activeDest)
	{
		return super.getRoute(activeDest);
	}

	public RouteEntry getRoute(Integer activeDest)
	{
		return super.getRoute(activeDest);
	}

	/**
	 * Remove a route from the table.
	 */
	public void removeRoute(RouteEntry r)
	{
		super.package_removeRoute(r);
	}

	/**
	 * Add a route to the table.
	 */
	public void addRoute(RouteEntry route) 
	{
		super.package_addRoute(route);
	}

	/**
	 * Add a route to the table.
	 */
	public void addRoute(int activeDest, int activeNext, String physNext)
		throws NoSuchNodeAddress, AddressFormatException
	{
		super.package_addRoute(new RouteEntry(activeDest, activeNext, physNext));
	}
}

