/*
 * ICMPFlow.Java
 *
 * Copyright (c) 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */
package JANOS_SERVICES_PKG.icmp;

import java.util.Hashtable;

import java.io.IOException;

import JANOS_NODEOS_PKG.Flow;
import JANOS_NODEOS_PKG.Node;
import JANOS_NODEOS_PKG.FlowRun;
import JANOS_NODEOS_PKG.Address;
import JANOS_NODEOS_PKG.InChannel;
import JANOS_NODEOS_PKG.OutChannel;
import JANOS_NODEOS_PKG.BufferHandle;
import JANOS_NODEOS_PKG.DemultiplexKey;
import JANOS_NODEOS_PKG.CommSpaceHandle;
import JANOS_NODEOS_PKG.AddressFormatException;
import JANOS_NODEOS_PKG.ProtocolSpecFormatException;
import JANOS_NODEOS_PKG.PacketAddressFormatException;

import JANOS_PROTOCOLS_PKG.ip.IPHeader;
import JANOS_PROTOCOLS_PKG.icmp.ICMPHeader;

public class ICMPFlow
	extends FlowRun
{
	private Flow flow;

	private Hashtable cachedRoutes = new Hashtable();
	private OutChannel defaultRoute;

	public Flow getFlow()
	{
		return this.flow;
	}

	private OutChannel createRoute(String myProtocol, String ethaddr)
	{
		OutChannel retval = null;

		try
		{
			retval = OutChannel.create(this.flow,
						   myProtocol,
						   ethaddr + ":0x800");
		}
		catch(ProtocolSpecFormatException e)
		{
			Node.println("Error: `"
				     + myProtocol
				     + "' Is not a valid protocol spec.");
		}
		catch(PacketAddressFormatException e)
		{
			Node.println("Error: `"
				     + ethaddr
				     + "' Is not a valid address spec.");
		}
		catch(IOException e)
		{
			Node.println("Error: `"
				     + ethaddr
				     + "' Is not a valid address spec.");
		}
		catch(Error e)
		{
			Node.println("Error: `"
				     + e
				     + "' Unknown error.");
		}
		return retval;
	}
	
	private void configure(String args[])
		throws AddressFormatException,
		       ProtocolSpecFormatException,
		       PacketAddressFormatException,
		       IOException
	{
		String str;
		int lpc;

		for( lpc = 0;
		     (str = Flow.lookupConfig("ICMPConfig:interface" + lpc,
					      "protocol")) != null;
		     lpc++ )
		{
			String protocolSpec = str;
			int address;

			str = Flow.lookupConfig("ICMPConfig:interface" + lpc,
						"address");
			address = Address.fromString(str);
			new ICMPPacketHandler(this, address, protocolSpec);
		}
	}

	PACKAGE OutChannel lookupRoute(String myProtocol, int address)
	{
		Integer i = new Integer(address);
		OutChannel retval;
		
		if( (retval = (OutChannel)this.cachedRoutes.get(i)) == null )
		{
			String str;

			str = Flow.lookupConfig("ICMPConfig:arp-table",
						Address.toString(address));
			if( str != null )
			{
				retval = this.createRoute(myProtocol, str);
				this.cachedRoutes.put(i, retval);
			}
		}
		return retval;
	}

	protected void run(Flow flow,
			   String args[],
			   CommSpaceHandle commSpaceHandle)
	{
		this.flow = flow;
		try
		{
			this.configure(args);
			Node.println("Finished configuring, waiting for packets");
		}
		catch(Exception e)
		{
			Node.println("Exception thrown during startup: " + e);
			Node.printStackTrace(e);
		}
		catch(Error e)
		{
			Node.println("Error thrown during startup: " + e);
			Node.printStackTrace(e);
		}
	}
	
}
