/*
 * UnknownPacketHandler.Java
 *
 * Copyright (c) 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */
package JANOS_SERVICES_PKG.icmp;

import java.io.IOException;

import JANOS_NODEOS_PKG.Node;
import JANOS_NODEOS_PKG.Flow;
import JANOS_NODEOS_PKG.InChannel;
import JANOS_NODEOS_PKG.DeliverFunc;
import JANOS_NODEOS_PKG.BufferHandle;
import JANOS_NODEOS_PKG.DemultiplexKey;
import JANOS_NODEOS_PKG.ProtocolSpecFormatException;
import JANOS_NODEOS_PKG.PacketAddressFormatException;

PACKAGE class UnknownPacketHandler
	implements DeliverFunc
{
	private static final int PACKET_SUMMARY_SIZE = 16;
	
	private ICMPFlow flow;
	private InChannel ic;
	
	PACKAGE UnknownPacketHandler(ICMPFlow flow,
				     String protocolSpec,
				     String addressSpec)
		throws ProtocolSpecFormatException,
		       PacketAddressFormatException,
		       IOException
	{
		DemultiplexKey dk;

		dk = new DemultiplexKey(Flow.currentFlow());
		this.flow = flow;
		this.ic = InChannel.create(Flow.currentFlow(),
					   dk,
					   protocolSpec,
					   addressSpec,
					   this);
		this.ic.addBuffers(10);
	}
	
	public void receive(BufferHandle bh, int payloadOffset)
	{
		int lpc, len = bh.getLength();
		
		Node.println("Received unknown packet:");
		if( len > PACKET_SUMMARY_SIZE )
			len = PACKET_SUMMARY_SIZE;
		for( lpc = 0; lpc < len; lpc++ )
		{
			String hex = Integer.toHexString(bh.getByte(lpc) &
							 0xff);
			
			if (hex.length() == 1)
				Node.print(" 0"+hex);
			else
				Node.print(" "+hex);
		}
	}
}
