/*
 * DoWhileCompilerBug.Java
 *
 * Copyright (c) 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */
package JANOS_TESTS_PKG;


import JANOS_NODEOS_PKG.Node;
import JANOS_NODEOS_PKG.Flow;

import JANOS_UTIL_PKG.FlowBase;

/**
 * Kaffe's KJC compiler has a bug where it cannot distinguish
 * correctly between overloaded methods put(byte), put(short), and
 * put(int).  The XDR code in ANTSR relies heavily on this
 * distinction.
 * <p>
 * Its pretty obvious if you have the bug.
 *
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu">Janos Ministry of Development</a>
 */
public class OverloadingCompilerBug
	extends FlowBase
{
	private int offset;

	public OverloadingCompilerBug()
	{
	}

	public void start()
	{
		this.check();
		Node.println("Checks passed.");
		Flow.currentFlow().destroy();
	}

	public void put(byte x)
	{
		offset+=1;
	}

	public void put(short y)
	{
		offset+=2;
	}

	public void put(int z)
	{
		offset+=4;
	}

	
	public void checkOffset(int supposedToBe)
	{
		if (offset != supposedToBe)
		{
			Node.println("Offset should be " +supposedToBe+ ", but is " +offset);
			this.flow.destroy();
		}
		
		Node.println("Offset is okay (" +offset+ ").");
	}
	
	public void check()
	{
		offset = 0;
		put((byte)0);  // offset += 1
		put((short)0); // offset += 2
		put((int)0);   // offset += 4
		checkOffset(7);
	}
}
