/*
 * WatchDog.Java
 *
 * Copyright (c) 1999 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package JANOS_TESTS_PKG;

/**
 * A simple watch dog thread.  It waits for a given length of time and then
 * interrupts the target thread, unless it gets interrupted beforehand.
 */
public class WatchDog extends Thread
{
	/**
	 * The amount of time to sleep before interrupting thread
	 */
	private long time;

	/**
	 * The thread to watch, and interrupt if time expires before we're
	 * interrupted
	 */
	private Thread thread;

	/**
	 * Construct a watch dog thread that watches a thread for ti time
	 *
	 * @param ti The length of time to sleep before interrupting the thread
	 * @param th The thread to watch
	 */
	public WatchDog(long ti, Thread th)
	{
		this.time = ti;
		this.thread = th;
	}

	/**
	 * Sleep for time milliseconds and then call interrupt on thread if
	 * we aren't interrupted before hand.  An interrupt called on our
	 * thread will cause make the thread quit quietly.
	 */
	public void run()
	{
		try
		{
			Thread.sleep(this.time);
			System.err.println("Watch dog timed out!");
			this.thread.interrupt();
			System.err.println("Interrupted thread "
					   + this.thread);
		}
		catch(InterruptedException e)
		{
		}
	}
}
