/*
 * DebugFlow.Java
 *
 * Copyright (c) 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */
package JANOS_TESTS_PKG.demo;

import JANOS_NODEOS_PKG.Flow;
import JANOS_NODEOS_PKG.Node;
import JANOS_NODEOS_PKG.InChannel;
import JANOS_NODEOS_PKG.DeliverFunc;
import JANOS_NODEOS_PKG.BufferHandle;
import JANOS_NODEOS_PKG.DemultiplexKey;
import JANOS_NODEOS_PKG.CommSpaceElement;

import java.io.IOException;

/**
 */
class DebugFlow
	extends SubFlow
	implements DeliverFunc
{
	private InChannel chan;
	private int iface;

	// controlle constructor
	DebugFlow()
	{
		super();
	}

	// controller contructor
	DebugFlow(int iface)
	{
		super(new CommSpaceElement[]{
			new CommSpaceElement(iface)
				});
	}

	void doRun(CommSpaceElement[] args)
	{
		this.iface = args[0].getInt();

		Node.println("START DEBUG FLOW on eth" +iface+ ".");

		Node.println("Installing listen channel on eth" +this.iface+ " (null dk)");
		try
		{
			this.chan = InChannel.create(Flow.currentFlow(),
						     new DemultiplexKey(Flow.currentFlow()),
						     "eth" + iface,
						     "0:0",
						     this);
		}
		catch (Exception e)
		{
			FATALEXCEPTION(e, "BAH!");
		}
		
		this.chan.addBuffers(10);
	}

	int lastLen = -1;
	short lastType = -1;
	int repeatCt = 0;

	public void receive(BufferHandle packet, int payloadOffset)
		throws IOException
	{
		int len = packet.getLength();
		short type = packet.getShort(13);

		if ((len == lastLen) && (type == lastType))
		{
			repeatCt++;
			if (repeatCt == 50)
			{
				Node.println("Got 50 of the last packet!");
				repeatCt = 0;
			}
		}
		else
		{
			if (repeatCt > 0)
				Node.println("Got "+repeatCt+ " of the last packet.");

			Node.print("Got packet on eth" +iface+ " debug flow: ");
			Node.print("offset=" +payloadOffset+ "; len=" +len+ "; ");
			Node.println("eth-type:" +Integer.toHexString(type)+ ".");
			lastLen = len;
			lastType = type;
			repeatCt = 0;
		}
		
		// recycle the packet
		this.chan.addBuffer(packet);
	}
	
	void doStartMisbehaving()
	{
		// This flow does not misbehave
		return;
	}
	
	void doStopMisbehaving()
	{
		// This flow does not misbehave
		return;
	}
}

