/*
 * Stats.Java
 *
 * Copyright (c) 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */
package JANOS_TESTS_PKG.demo;

import JANOS_NODEOS_PKG.Flow;
import JANOS_NODEOS_PKG.Node;
import JANOS_NODEOS_PKG.Debug;
import JANOS_NODEOS_PKG.Address;
import JANOS_NODEOS_PKG.FlowRun;
import JANOS_NODEOS_PKG.MemSpec;
import JANOS_NODEOS_PKG.TimeSpec;
import JANOS_NODEOS_PKG.FlowStats;
import JANOS_NODEOS_PKG.ChanStats;
import JANOS_NODEOS_PKG.InChannel;
import JANOS_NODEOS_PKG.OutChannel;
import JANOS_NODEOS_PKG.Credentials;
import JANOS_NODEOS_PKG.DeliverFunc;
import JANOS_NODEOS_PKG.NetworkSpec;
import JANOS_NODEOS_PKG.BufferHandle;
import JANOS_NODEOS_PKG.FlowStartSpec;
import JANOS_NODEOS_PKG.DemultiplexKey;
import JANOS_NODEOS_PKG.ThreadPoolSpec;
import JANOS_NODEOS_PKG.CommSpaceHandle;
import JANOS_NODEOS_PKG.CommSpaceElement;
import JANOS_NODEOS_PKG.ProtocolSpecFormatException;
import JANOS_NODEOS_PKG.PacketAddressFormatException;

import JANOS_SERVICES_PKG.arp.ARPHandler;

import JANOS_PROTOCOLS_PKG.ip.IPHeader;
import JANOS_PROTOCOLS_PKG.udp.UDPHeader;

PACKAGE class Stats
	implements Runnable
{
	private Flow flow;
	private Main root;
	private BufferHandle statsBuffer;
	private int bufferOffset;
	private IPHeader ip;
	private UDPHeader udp;
	private FlowStats fs = new FlowStats();
	private ChanStats cs = new ChanStats();
	private TimeSpec ts = new TimeSpec();
	private int INTERVAL = 1000;
	private int INTERFACE = 0;
	private short DEST_PORT = 8008;
	private OutChannel oc;
	
	public Stats(Flow flow, Main root)
	{
		this.flow = flow;
		this.root = root;
	}

	public void configure()
		throws Exception
	{
		byte ethaddr[] = { 0, 0, 0, 0, 0, 0 };
		byte ipaddr[] = { 0, 0, 0, 0 };
		StringBuffer sb = new StringBuffer();
		int lpc, src = 0, dst = 0;
		String str, addrSpec;

		str = Flow.lookupConfig("ControlConfig", "address");
		if( str != null )
			src = Address.fromString(str);
		str = Flow.lookupConfig("ControlConfig:stats", "destination");
		if( str != null )
			dst = Address.fromString(str);
		else
			throw new Exception("No destination specified");
		str = Flow.lookupConfig("ControlConfig:stats",
					"destination-port");
		if( str != null )
			DEST_PORT = Short.parseShort(str);
		str = Flow.lookupConfig("ControlConfig:stats", "interval");
		if( str != null )
			INTERVAL = Integer.parseInt(str);
		str = Flow.lookupConfig("ControlConfig:stats", "interface");
		if( str != null )
			INTERFACE = Integer.parseInt(str);
		str = Flow.lookupConfig("ControlConfig:stats", "destination-eth");
		if( str != null )
		{
			addrSpec = str + ":0x800";
		}
		else
		{
			ipaddr[0] = (byte)((dst >> 24) & 0xFF);
			ipaddr[1] = (byte)((dst >> 16) & 0xFF);
			ipaddr[2] = (byte)((dst >>  8) & 0xFF);
			ipaddr[3] = (byte)((dst      ) & 0xFF);
			if( ARPHandler.arpfor(INTERFACE,
					      ipaddr,
					      ethaddr) != 0 )
			{
				throw new Exception("Couldn't arp "
						    + Address.toString(dst));
			}
			Node.print("arpfor = ");
			for( lpc = 0; lpc < ethaddr.length; lpc++ )
			{
				if( lpc > 0 )
					Node.print(":");
				Node.print(Integer.
					   toHexString(ethaddr[lpc] & 0xff));
			}
			Node.println(".");
			for( lpc = 0; lpc < ethaddr.length; lpc++ )
			{
				String num;

				num = Integer.toHexString(ethaddr[lpc] & 0xff);
				if( num.length() == 1 )
					sb.append("0" + num);
				else
					sb.append(num);
			}
			Node.println("ARP for "
				     + (ipaddr[0] & 0xff)
				     + "."
				     + (ipaddr[1] & 0xff)
				     + "."
				     + (ipaddr[2] & 0xff)
				     + "."
				     + (ipaddr[3] & 0xff)
				     + " is "
				     + sb.toString());
			addrSpec = "0x" + sb.toString() + ":0x800";
		}
		oc = OutChannel.create(this.flow,
				       new NetworkSpec(),
				       "if" + INTERFACE,
				       addrSpec);
		ip = new IPHeader((byte)0,
				  (short)0x23,
				  (short)0,
				  (byte)16,
				  IPHeader.PROTO_UDP,
				  src,
				  dst);
		udp = new UDPHeader((short)8887,
				    DEST_PORT,
				    (short)this.statsLength());
		statsBuffer = BufferHandle.create(0,
						  ip.getLength()
						  + udp.getLength()
						  + udp.getBodyLength());
		bufferOffset = this.primeStats(statsBuffer);

		Node.println("Stats config:");
		Node.println("  Channel: "
			     + "if" + INTERFACE
			     + " "
			     + addrSpec);
		Node.println("  Destination: " + Address.toString(dst));
		Node.println("  Port: " + DEST_PORT);
		Node.println("  Interval: " + INTERVAL);
	}
	
	private void marshalStats(BufferHandle bh, int offset)
	{
		final int STATS_VERSION_JAVA = 0x4a617661;
		int ipOffset = offset;

		ip.marshal(bh, 0);
		ip.newID();
		offset += udp.getLength();
		bh.setInt(offset, STATS_VERSION_JAVA);
		offset += 4;
		TimeSpec.getTime(ts);
		offset += ts.marshal(bh, offset);
		this.root.forwarder.getHandle().getStats(fs);
		offset += fs.marshal(bh, offset);
		this.root.netHog.getHandle().getStats(fs);
		offset += fs.marshal(bh, offset);
		this.root.cpuHog.getHandle().getStats(fs);
		offset += fs.marshal(bh, offset);
		this.root.memHog.getHandle().getStats(fs);
		offset += fs.marshal(bh, offset);
		this.root.forwarder.getHandle().getOutChannelStats(0, cs);
		offset += cs.marshal(bh, offset);
		this.root.netHog.getHandle().getOutChannelStats(0, cs);
		offset += cs.marshal(bh, offset);
		udp.marshal(bh, ipOffset);
	}

	private int primeStats(BufferHandle bh)
	{
		int retval;

		retval = ip.marshal(bh, 0);
		return retval;
	}
	
	private int statsLength()
	{
		int retval;

		retval = 4;
		retval += ts.getLength();
		retval += fs.getLength();
		retval += fs.getLength();
		retval += fs.getLength();
		retval += fs.getLength();
		retval += cs.getLength();
		retval += cs.getLength();
		retval += 32; // XXX mempool stats
		return retval;
	}

	public void run()
	{
		IPHeader myIP = new IPHeader();
		UDPHeader myUDP = new UDPHeader();
		int offset;
		
		while( true )
		{
			try
			{
				Thread.sleep(INTERVAL);
				this.marshalStats(statsBuffer, bufferOffset);
#if 0
				offset = myIP.unmarshal(statsBuffer, 0);
				offset += myUDP.unmarshal(statsBuffer, offset);
				Node.println("Sending:\n"
					     + myIP
					     + "\n"
					     + myUDP);
#endif
				oc.send(statsBuffer);
			}
			catch(Throwable e)
			{
				Node.println("Caught exception? " + e);
				Node.printStackTrace(e);
			}
		}
	}
}
