/*
 * PacketDump.Java
 *
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */
package JANOS_TESTS_PKG.envs;

import JANOS_NODEOS_PKG.Node;
import JANOS_NODEOS_PKG.Flow;
import JANOS_NODEOS_PKG.InChannel;
import JANOS_NODEOS_PKG.DeliverFunc;
import JANOS_NODEOS_PKG.BufferHandle;
import JANOS_NODEOS_PKG.DemultiplexKey;
import JANOS_NODEOS_PKG.ProtocolSpecFormatException;
import JANOS_NODEOS_PKG.PacketAddressFormatException;
import JANOS_UTIL_PKG.FlowBase;

import java.io.IOException;

/**
 * Java equivalent of the Moab "packetdump" environment.  Prints every
 * packet received on every interface.
 */
public class PacketDump
	extends FlowBase
{
	/**
	 * run for this many seconds before shutting down.
	 */
 	private final int RUN_FOR = 60;

	private final int MAX_PACKET_DUMP_LEN = 32;

	public PacketDump()
	{
	}

	private class DieEEDie
		extends Thread
	{
		private Flow flow;

		public DieEEDie(Flow flow)
		{
			this.flow = flow;
		}

		public void run()
		{
			try
			{
				Thread.sleep(RUN_FOR * 1000);
			}
			catch (InterruptedException ie)
			{
				// nothing 
			}

			Node.println("Time's up.  Nuking myself.");
			die();
		}
		
		public void die()
		{
			// XXX this.flow.destroy();
			Node.shutdown();
		}
	}

	private class DumpChannel
		implements DeliverFunc
	{
		InChannel chan;
		int id;

		public DumpChannel(int id)
			throws ProtocolSpecFormatException, PacketAddressFormatException, IOException
		{
			Flow f = Flow.currentFlow();
			DemultiplexKey dk;

			this.id = id;

			/* Null demux key matches everything. */
			dk = new DemultiplexKey(f);
			
			this.chan = InChannel.create(Flow.currentFlow(),
						     dk,
						     "eth" + this.id,
						     "0:0",
						     this);

			this.chan.addBuffers(10);
		}

		public void receive(BufferHandle packet, int payloadOffset)
			throws IOException
		{
			int l = packet.getLength();

			Node.print(this.id+ " [" +l+"]:");
		PRINTPACKET:
			for (int i = 0; i < l; i++)
			{
				String hex = Integer.toHexString(packet.getByte(i) & 0xff);
				if (hex.length() == 1)
					Node.print(" 0"+hex);
				else
					Node.print(" "+hex);

				if (i == MAX_PACKET_DUMP_LEN)
					break PRINTPACKET;
			}
			Node.println("");

			// recycle the buffer
			this.chan.addBuffer(packet);
		}
	}

	public void start()
	{
		DieEEDie die = new DieEEDie(Flow.currentFlow());
		Node.println("PacketDump" + ": initialization");

		try
		{
			setupChannels();
			Node.println("All channels are 'GO'.");
			die.start();
		}
		catch (Exception e)
		{
			Node.println("Oops: " +e);
			Node.printStackTrace(e);
			die.die();
		}
	}

	public void setupChannels()
		throws Exception
	{
		/* XXX get interface info! */

		int nif = 5;
		
		for (int i = 0; i < nif; i++)
		{
			new DumpChannel(i);
		}
	}
	
}
