/*
 * CommQueueTester.Java
 *
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package JANOS_TESTS_NODEOS_PKG;

import JANOS_TESTS_PKG.*;
import JANOS_NODEOS_PKG.*;

public class CommQueueTester extends Tester
{
	private TestFailedException consumerException = null;
	private final int exitValue = 10;

	/**
	 * Runnable class that consumes data
	 */
	private class Consumer implements Runnable
	{
		private CommQueuePuller cq;
		
		public Consumer(CommQueuePuller cq)
		{
			this.cq = cq;
		}
		
		public void run()
		{
			CommSpaceElement elem;
			
			try
			{
				boolean done = false;
				
				while( !done )
				{
					elem = cq.waitToTake();
					if( elem.getInt() == exitValue )
						done = true;
				}
			}
			catch(InterruptedException e)
			{
				CommQueueTester cqt =
					CommQueueTester.this;
				
				cqt.consumerException =
					new TestFailedException(cqt);
			}
		}
	}
	
	public void testUnit() throws TestFailedException
	{
		final CommQueuePuller pull = new CommQueuePuller();
		final CommQueuePusher push = new CommQueuePusher(pull);
		java.lang.Thread watchDog;
		CommSpaceElement cse;
		int lpc;

		this.enterSection(ENV_NONE, "Queue operations");

		/* enqueue some elements to be taken */
		int elements[] =
		{
			1,
			2,
			3,
			4
		};
		for( lpc = 0; lpc < elements.length; lpc++ )
		{
			cse = new CommSpaceElement(elements[lpc]);
			push.enqueue(cse);
		}

		/* take the elements, use watch dog so we don't stall */
		watchDog = new WatchDog(10000,
					java.lang.Thread.currentThread());
		watchDog.start();
		try
		{
			for( lpc = 0; lpc < elements.length; lpc++ )
			{
				cse = pull.waitToTake();
				this.test(cse.getInt() == elements[lpc]);
			}
		}
		catch(InterruptedException e)
		{
			throw new TestFailedException(this);
		}
		watchDog.interrupt();

		java.lang.Thread consumer;

		/* start a consumer thread and enqueue some elements for it */
		consumer = new java.lang.Thread(new Consumer(pull));
		try
		{
			consumer.start();
			for( lpc = 0; lpc <= exitValue; lpc++ )
			{
				/* sleep so the consumer can do something */
				java.lang.Thread.sleep(50);
				push.enqueue(new CommSpaceElement(lpc));
			}
			consumer.join(1000);
			if( consumer.isAlive() )
			{
				consumer.interrupt();
				throw new TestFailedException(this);
			}
		}
		catch(InterruptedException e)
		{
			consumer.interrupt();
			throw new TestFailedException(this);
		}
		if( this.consumerException != null )
			throw this.consumerException;
	}
}
