/*
 * FlowBase.Java
 *
 * Copyright (c) 1999, 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */
package JANOS_UTIL_PKG;

import JANOS_NODEOS_PKG.Flow;
import JANOS_NODEOS_PKG.FlowRun;
import JANOS_NODEOS_PKG.BufferHandle;
import JANOS_NODEOS_PKG.CommSpaceHandle;

/**
 * A protocol should override FlowBase so that a flow can
 * be created for it.  When the flow is created, the overridden start()
 * method is called.
 * <p>
 * The flow and args variables are guaranteed to
 * be set before start() is called, but may not be set when
 * the constructor is called.
 */
public abstract class FlowBase extends FlowRun
{
	public Flow flow;
	public String[] args;
	public CommSpaceHandle commSpaceHandle;

	protected FreeList bufferHandleFreeList = new FreeList();

	/**
	 * This sets the flow, and args fields and then
	 * calls start().
	 */
	public void run(Flow flow,
			String[] args,
			CommSpaceHandle commSpaceHandle)
	{
		this.flow = flow;
		this.args = args;
		this.commSpaceHandle = commSpaceHandle;
		start();
	}

	public abstract void start();
}

