/*
 * PlainPacketSegment.Java
 *
 * Copyright (c) 2000 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Janos Java NodeOS software.
 * 
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */
package JANOS_UTIL_PKG;

import JANOS_NODEOS_PKG.BufferHandle;

/**
 * A PlainPacketSegment implements the PacketSegment interface for an array
 * of bytes.
 */
public class PlainPacketSegment
	implements PacketSegment
{
	/**
	 * The array of bytes that make up the packet segment.
	 */
	private byte body[];

	/**
	 * @param body The array of bytes that should make up the segment.
	 */
	public PlainPacketSegment(byte body[])
	{
		this.body = body;
	}

	/**
	 * @param length The length of the expected byte array.
	 */
	public PlainPacketSegment(int length)
	{
		this.body = new byte[length];
	}

	/**
	 * @return The length of the byte array.
	 */
	public int getLength()
	{
		return this.body.length;
	}

	/**
	 * @return The byte array that makes up this segment.
	 */
	public byte[] getBody()
	{
		return this.body;
	}

	/**
	 * @param offset The offset into the body.
	 * @return The byte at the given offset in the body of the segment.
	 */
	public byte getByte(int offset)
	{
		return this.body[offset];
	}

	/**
	 * @param offset The offset into the body.
	 * @param val The byte at the given offset in the body of the segment.
	 */
	public void setByte(int offset, byte val)
	{
		this.body[offset] = val;
	}
	
	/**
	 * @param offset The offset into the body.
	 * @return The short at the given offset in the body of the segment.
	 */
	public short getShort(int offset)
	{
		return (short)(((this.body[offset++] & 0xff) << 8)
			       | (this.body[offset] & 0xff));
	}

	/**
	 * @param offset The offset into the body.
	 * @param val The short at the given offset in the body of the segment.
	 */
	public void setShort(int offset, short val)
	{
		this.body[offset++] = (byte)((val >>> 8) & 0xff);
		this.body[offset++] = (byte)((val      ) & 0xff);
	}

	/**
	 * @param offset The offset into the body.
	 * @return The int at the given offset in the body of the segment.
	 */
	public int getInt(int offset)
	{
		return (((this.body[offset++] & 0xff) << 24)
			| ((this.body[offset++] & 0xff) << 16)
			| ((this.body[offset++] & 0xff) << 8)
			| (this.body[offset] & 0xff));
	}
	
	/**
	 * @param offset The offset into the body.
	 * @param val The int at the given offset in the body of the segment.
	 */
	public void setInt(int offset, int val)
	{
		this.body[offset++] = (byte)((val >>> 24) & 0xff);
		this.body[offset++] = (byte)((val >>> 16) & 0xff);
		this.body[offset++] = (byte)((val >>>  8) & 0xff);
		this.body[offset++] = (byte)((val       ) & 0xff);
	}
	
	public int marshal(BufferHandle bh, int offset)
	{
		bh.setBytes(offset, this.body, 0, this.body.length);
		return offset + this.body.length;
	}
	
	public int unmarshal(BufferHandle bh, int offset)
		throws InvalidPacketException
	{
		bh.getBytes(offset, this.body, 0, this.body.length);
		return offset + this.body.length;
	}

	public String toString()
	{
		return "[PlainPacketSegment body="
			+ this.body
			+ "]";
	}
}
